﻿// Copyright (c) 2020 Tudor Stanciu

using System.Linq;
using Tuitio.Application.Extensions;
using Tuitio.Application.Services.Abstractions;
using Tuitio.Domain.Abstractions;
using Tuitio.Domain.Entities;
using Tuitio.Domain.Models;

namespace Tuitio.Application.Services
{
    internal class TokenService : ITokenService
    {
        private readonly IConfigProvider _configProvider;

        public TokenService(IConfigProvider configProvider)
        {
            _configProvider=configProvider;
        }

        public Token GenerateToken(AppUser user)
        {
            var token = new Token(_configProvider.Token.ValidityInMinutes);
            var claims = user.Claims?.ToDictionary();
            var userRoles = user.GetUserRoles().Select(z => z.UserRoleCode);
            var userGroups = user.UserGroups?.Select(z => z.UserGroup.UserGroupCode);
            token.SetUserData(user.UserId, user.UserName, user.FirstName, user.LastName, user.Email, user.SecurityStamp, claims, userRoles, userGroups);
            return token;
        }
    }
}
